#include "stdafx.h"
#include <memory.h>
#include "AvSyncConvertAviWrapper.h"

#define DEFAULT_AVIVFRAMEWIDTH    352   // Default AVI video frame width
#define DEFAULT_AVIVFRAMEHEIGHT   240   // Default AVI video frame width
#define DEFAULT_AVIVBITCOUNT      24    // Default AVI video bit count
#define DEFAUL_AVIVFRAMERATE      15    // Default AVI video frame rate
#define DEFAULT_AVIACHANNELS      1     // Default AVI audio channels
#define DEFAULT_AVIASAMPLEPERSEC  8000  // Default AVI audio samples per second
#define DEFAULT_AVIABITSPERSAMPLE 16    // Default AVI audio bits per sample


// Function     : CAvSyncConvertAviWrapper::CAvSyncConvertAviWrapper
// Description  : Constructor
// Parameter    : 
//              | [in] hDlg     @Dialog window handle
//              | [in] hDisplay @Display window handle
//              | [in] hBMP     @Bitmap handle
//              | [in] pfStatus         @Status callback function
//              | [in] pfDisplay        @Display callback function
//              | [in] pfAviStatus      @Avi status callback function
//              | [in] pfAviCaption     @Avi display callback function
//              | [in] dwStatusContext  @Status context
//              | [in] dwDisplayContext @Display context
//              | [in] dwStatusContext  @Avi status context
//              | [in] dwDisplayContext @Avi display context
// Return value : N/A
//
CAvSyncConvertAviWrapper::CAvSyncConvertAviWrapper(HWND hDlg, HWND hDisplay, HBITMAP hBMP, LPSTATUSCALLBACK pfStatus, 
												                                           LPDISPLAYCALLBACK pfDisplay, 
																						   LPSTATUSCALLBACK pfAviStatus, 
																						   LPDISPLAYCALLBACK pfAviCaption, 
																						   DWORD dwStatusContext, 
																						   DWORD dwDisplayContext,
																						   DWORD dwAviStatusContext, 
																						   DWORD dwAviCaptionContext)
{
	m_hAvSync          = NULL;
	m_hPlaybackChannel = NULL;
	m_hAviChannel      = NULL;
	
	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.hDlg     = hDlg;
	m_tParam.hDisplay = hDisplay;
	m_tParam.hBMP     = hBMP;

	m_tParam.pfStatus     = pfStatus;
	m_tParam.pfDisplay    = pfDisplay;
	m_tParam.pfAviStatus  = pfAviStatus;
	m_tParam.pfAviCaption = pfAviCaption;
	
	m_tParam.dwStatusContext     = dwStatusContext;
	m_tParam.dwDisplayContext    = dwDisplayContext;
	m_tParam.dwAviStatusContext  = dwAviStatusContext;
	m_tParam.dwAviCaptionContext = dwAviCaptionContext;

	InitialModule();  // Initialize AvSynchronizer, create playback channel and AVI channel
}

// Function     : CAvSyncConvertAviWrapper::~CAvSyncConvertAviWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CAvSyncConvertAviWrapper::~CAvSyncConvertAviWrapper()
{
	ReleaseModule();  // Delete playback channel and AVI channel, release AvSynchronizer
}

// Function     : CAvSyncConvertAviWrapper::InitialModule
// Description  : Initialize AvSynchronizer, create playback channel and AVI channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::InitialModule()
{	
	SCODE scRet;

	// STEP 1: Initialize AvSynchronizer
	scRet = AvSynchronizer_InitialEx(&m_hAvSync, NULL, NULL, m_tParam.hDlg, AUDIOOUT_FOCUS_NORMAL, 0, AVSYNCHRONIZER_VERSION, 0, RGB(0, 0, 0));

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InitialEx fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	// STEP 2: Create playback channel and AVI chanenl
	if	(m_hAvSync != NULL)
	{
		scRet = CreateChannel(); 

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}

		scRet = CreateAVIChannel();

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::ReleaseModule
// Description  : Delete playback channel and AVI channel, release AvSynchronizer
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::ReleaseModule()
{
	SCODE scRet;
	
	if	(m_hAvSync != NULL)
	{
		// STEP 1: Delete playback channel and AVI channel
		if	(m_hPlaybackChannel != NULL)
		{
			scRet = DeleteChannel(); 

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		if	(m_hAviChannel != NULL)
		{
			scRet = DeleteAVIChannel();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release AvSynchronizer
		scRet = AvSynchronizer_Release(&m_hAvSync);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_Release fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::CreateChannel
// Description  : Create playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::CreateChannel()
{	
	// STEP 1: Set channel options
	TPBCHOPTION tPBCHOption;

	memset(&tPBCHOption, 0, sizeof(tPBCHOption));

	tPBCHOption.dwFlags = PBCH_STATUSCB | PBCH_DISPLAYCB | PBCH_STATUSCONTEXT | PBCH_DISPLAYCONTEXT | PBCH_ALLBORDER | PBCH_BITMAP;

	tPBCHOption.pfStatus  = m_tParam.pfStatus;
	tPBCHOption.pfDisplay = m_tParam.pfDisplay; 

	tPBCHOption.dwStatusContext  = m_tParam.dwStatusContext;
	tPBCHOption.dwDisplayContext = m_tParam.dwDisplayContext;
	
	tPBCHOption.dwTopBorderSize    = 20;
	tPBCHOption.dwLeftBorderSize   = 0;
	tPBCHOption.dwBottomBorderSize = 0;
	tPBCHOption.dwRightBorderSize  = 0;

	tPBCHOption.hBMP     = m_tParam.hBMP;
	tPBCHOption.hDisplay = m_tParam.hDisplay;
	
	SCODE scRet;

	// STEP 2: Create playback channel
	scRet = AvSynchronizer_CreatePlaybackChannel(m_hAvSync, &m_hPlaybackChannel, tPBCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreatePlaybackChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}
	
	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::DeleteChannel
// Description  : Delete playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::DeleteChannel()
{
	SCODE scRet;

	// Delete playback channel 
	scRet = AvSynchronizer_DeleteChannel(m_hAvSync, &m_hPlaybackChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::Play
// Description  : Start playback channel and AVI channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::Play()
{
	SCODE scRet;
	
	// Start playback channel
	if	(m_hPlaybackChannel != NULL)
	{	
		scRet = AvSynchronizer_StartChannel(m_hPlaybackChannel, 0);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StartChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}
	
	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::Stop
// Description  : Stop playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::Stop()
{	
	SCODE scRet;
	
	// Stop playback channel
	if	(m_hPlaybackChannel != NULL)
	{	
		scRet = AvSynchronizer_StopChannel(m_hPlaybackChannel);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StopChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}

		// Show bitmap and reset play speed to normal
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		DWORD dwNormalPlaySpeed = 1;

		tUpdatePBCHOption.dwFlags = UPCH_SHOW_BMP | UPCH_SETSPEED;
		tUpdatePBCHOption.dwFast  = dwNormalPlaySpeed;
		tUpdatePBCHOption.dwSlow  = dwNormalPlaySpeed;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::Pause
// Description  : Pause or Resume playback channel
// Parameter    :
//              | [in] bPause @TRUE : pause, FALSE : resume
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::Pause(BOOL bPause)
{
	SCODE scRet;
	
	if	(m_hPlaybackChannel != NULL)
	{	
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		tUpdatePBCHOption.dwFlags = UPCH_PAUSE;
		tUpdatePBCHOption.bPause  = bPause;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::Forward
// Description  : Change play speed of playback channel
// Parameter    :
//              | [in] dwFast @Multiple of normal play speed
//              | [in] dwSlow @Divisor of normal play speed
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::Forward(DWORD dwFast, DWORD dwSlow)
{
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{	
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		tUpdatePBCHOption.dwFlags = UPCH_SETSPEED;
		tUpdatePBCHOption.dwFast  = dwFast;
		tUpdatePBCHOption.dwSlow  = dwSlow;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::NextFrame
// Description  : Jump to next frame. The playback channel must be paused previously!
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::NextFrame()
{
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{	
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		tUpdatePBCHOption.dwFlags = PBCH_NEXTFRAME;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::UpdatePlaybackChannelMediaType
// Description  : Update channel media type
// Parameter    :
//              | [in] dwMediaType @Media type
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::UpdatePlaybackChannelMediaType(DWORD dwMediaType)
{
	TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

	memset(&tUpdatePBCHOption, 0, sizeof(TUPDATEPBCHANNELOPTION));

	tUpdatePBCHOption.dwFlags = UPCH_MEDIATYPE;
	tUpdatePBCHOption.tMediaType = EMEDIATYPE(dwMediaType);

	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{
		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::InputDataPacketToChannel
// Description  : Input the parsed data packet info to playback channel
// Parameter    :
//              | [in] ptPacketInfo @Input media data packet info
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::InputDataPacketToChannel(TMediaDataPacketInfoV3 *ptPacketInfoV3)
{
	// Notice!
	// If the internal buffer of "PLAYBACK CHANNEL" is full, it will block the calling thread by default. 
	// User can call AvSynchronizer_SetChannelOption with SETCH_NOT_BLOCK_WHEN_QUEUE_FULL to enable or disable this function.
	
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{
		TMediaDataPacketInfo *ptPacketInfo = (TMediaDataPacketInfo *)ptPacketInfoV3;  // Cast "ptPacketInfoV3" to (TMediaDataPacketInfo *)

		// Input the parsed data packet info to playback channel
		scRet = AvSynchronizer_InputPlaybackMediaFrame(m_hPlaybackChannel, ptPacketInfo);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputPlaybackMediaFrame fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::WaitAVQueueEmpty
// Description  : Wait until the audio and video elements in AvSynchronizer's queue are all played 
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::WaitAVQueueEmpty()
{
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{
		TAVSYNCQUEUESTATUS tAvSyncQueueStatus;

		// Wait until all audio and video elements are played.
		// When all elements are played, the audio buffer queue is unused now, hence the dwAudioQueueElements would be equal to dwAudioQueueSize.
		// On the other hand, the video buffer will keep one unit to redraw the last frame, hence the dwVideoQueueElements = dwVideoQueueSize - 1.
		//
		do{
			Sleep(20);

			memset(&tAvSyncQueueStatus, 0, sizeof(tAvSyncQueueStatus));

			// Get remaining audio and video elements in queue
			scRet = AvSynchronizer_GetRemainingQueueElementCount(m_hPlaybackChannel, &tAvSyncQueueStatus);

			if	(IS_FAIL(scRet))
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_GetRemainingQueueElementCount fail! (ERROR: %X)\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return S_FAIL;
			}
		
		}while	((tAvSyncQueueStatus.dwAudioQueueElements != tAvSyncQueueStatus.dwAudioQueueSize) || (tAvSyncQueueStatus.dwVideoQueueElements != tAvSyncQueueStatus.dwVideoQueueSize - 1));
	}
	
	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::InputDataPacketToAVIChannel
// Description  : Input the parsed data packet info to AVI channel
// Parameter    :
//              | [in] ptAviPacketInfoV3 @Input media data packet info
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::InputDataPacketToAVIChannel(TMediaDataPacketInfoV3 *ptPacketInfoV3)
{
	SCODE scRet;

	if	(m_hAviChannel != NULL)
	{
		TMediaDataPacketInfo *ptPacketInfo = (TMediaDataPacketInfo *)ptPacketInfoV3;  // Cast "ptPacketInfoV3" to (TMediaDataPacketInfo *)

		// Input the parsed data packet info to AVI channel
		scRet = AvSynchronizer_InputAVIMedia(m_hAviChannel, ptPacketInfo);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputAVIMedia fail! (ERROR: %X)", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::CreateAVIChannel
// Description  : Create AVI channel  
// Parameter    :
//              | [in] pfAviStatus      @Avi status callback function
//              | [in] pfAviCaption     @Avi display callback function
//              | [in] dwStatusContext  @Status context
//              | [in] dwDisplayContext @Display context
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::CreateAVIChannel()
{
	SCODE scRet;

	CoInitialize(NULL);  // AVI converter needs COM object
	
	// STEP 1: Set channel options
	TAVICHOPTION tAviCHOption;

	tAviCHOption.pfStatus  = m_tParam.pfAviStatus;
	tAviCHOption.pfCaption = m_tParam.pfAviCaption;

	tAviCHOption.dwStatusContext  = m_tParam.dwAviStatusContext;
	tAviCHOption.dwCaptionContext = m_tParam.dwAviCaptionContext;
	
	// STEP 2: Create AVI channel
	scRet = AvSynchronizer_CreateAVIChannel(m_hAvSync, &m_hAviChannel, &tAviCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreateAVIChannel fail! (ERROR: %X)", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::DeleteAVIChannel
// Description  : Delete AVI channel  
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::DeleteAVIChannel()
{
	SCODE scRet;

	// Delete AVI channel
	scRet = AvSynchronizer_DeleteAVIChannel(m_hAvSync, &m_hAviChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteAVIChannel fail! (ERROR: %X)", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::StartAVIChannel
// Description  : Start AVI channel  
// Parameter    : 
//              | [in] dwAviFrameWidth  @Frame width of Avi file
//              | [in] dwAviFrameHeight @Frame height of Avi file
//              | [in] dwAviFrameRate   @Frame rate of Avi file
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::StartAVIChannel(char *pszSaveAviDirectoyPath)
{
	// STEP 1: Set channel options
	TAVICHOPTION2 tAviCHOption2;

	memset(&tAviCHOption2, 0, sizeof(tAviCHOption2));
	
	// If the flag "FIXED_VINFO_AT_FIRST_FRAME" is set, the video size of the output AVI file would be determined by the first of the video frame.
	//If this flag is not set, users need to give the wanted video size when start the channel
	//
	tAviCHOption2.tAviVInfo.dwWidth    = DEFAULT_AVIVFRAMEWIDTH;   // 352
	tAviCHOption2.tAviVInfo.dwHeight   = DEFAULT_AVIVFRAMEHEIGHT;  // 240
	tAviCHOption2.tAviVInfo.dwBitCount = DEFAULT_AVIVBITCOUNT;     // 24

	// If the flag "FIXED_AINFO_AT_FIRST_FRAME" is set, the audio sample frequency of the output AVI file would be determined by the first of the audio frame.
	// If this flag is not set, users need to give the wanted audio sampling frequency when start the channel.
	//
	tAviCHOption2.tAviAInfo.dwChannels      = DEFAULT_AVIACHANNELS;       // 1
	tAviCHOption2.tAviAInfo.dwSamplesPerSec = DEFAULT_AVIASAMPLEPERSEC;   // 8000
	tAviCHOption2.tAviAInfo.dwBitsPerSample = DEFAULT_AVIABITSPERSAMPLE;  // 16

	tAviCHOption2.dwFrameRate = DEFAUL_AVIVFRAMERATE;  // 15

	tAviCHOption2.bVideoEnable = TRUE;
	tAviCHOption2.bAudioEnable = TRUE;	

	sprintf_s(tAviCHOption2.szFileName, sizeof(tAviCHOption2.szFileName), "%s\\Decode.avi", pszSaveAviDirectoyPath);

	tAviCHOption2.tBorder.dwHeader = 20;
	tAviCHOption2.tBorder.dwFooter = 0;
	tAviCHOption2.tBorder.dwLeft   = 0;
	tAviCHOption2.tBorder.dwRight  = 0;

	SCODE scRet;

	// STEP 2: Start AVI channel
	if	(m_hAviChannel != NULL)
	{
		scRet = AvSynchronizer_StartAVIChannel(m_hAviChannel, &tAviCHOption2);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StartAVIChannel fail! (ERROR: %X)", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::StopAVIChannel
// Description  : Stop AVI channel  
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::StopAVIChannel()
{
	SCODE scRet;

	if	(m_hAviChannel != NULL)
	{	
		// Stop AVI channel
		scRet = AvSynchronizer_StopAVIChannel(m_hAviChannel);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StopAVIChannel fail! (ERROR: %X)", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	CoUninitialize();

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::SetVideoCompressorParam
// Description  : Set video compressor parameters  
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::SetVideoCompressorParam()
{
	SCODE scRet;

	TAVIVINFO tAviInfo;

	tAviInfo.dwWidth    = DEFAULT_AVIVFRAMEWIDTH;   // 352
    tAviInfo.dwHeight   = DEFAULT_AVIVFRAMEHEIGHT;  // 240
    tAviInfo.dwBitCount = DEFAULT_AVIVBITCOUNT;     // 24

	if	(m_hAviChannel != NULL)
	{
		scRet = AvSynchronizer_ChooseVideoCompressor(m_hAvSync, m_tParam.hDlg, "Choose video compressor", &tAviInfo);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_ChooseVideoCompressor fail! (ERROR: %X)", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncConvertAviWrapper::SetAudioCompressorParam
// Description  : Set audio compressor parameters  
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncConvertAviWrapper::SetAudioCompressorParam()
{
	SCODE scRet;

	TAVIAINFO tAviInfo;

	tAviInfo.dwChannels      = DEFAULT_AVIACHANNELS;       // 1
	tAviInfo.dwSamplesPerSec = DEFAULT_AVIASAMPLEPERSEC;   // 8000
	tAviInfo.dwBitsPerSample = DEFAULT_AVIABITSPERSAMPLE;  // 16

	if	(m_hAviChannel != NULL)
	{
		scRet = AvSynchronizer_ChooseAudioCompressor(m_hAvSync, m_tParam.hDlg, "Choose audio compressor", &tAviInfo);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_ChooseAudioCompressor fail! (ERROR: %X)", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}
