#include "stdafx.h"
#include <memory.h>
#include "AvSyncLiveCustomDrawWrapper.h"
                       

// Function     : CAvSyncLiveCustomDrawWrapper::CAvSyncLiveCustomDrawWrapper
// Description  : Constructor
// Parameter    : 
//              | [in] hDlg             @Dialog window handle
//              | [in] hDisplay         @Display window handle
//              | [in] hBMP             @Bitmap handle
//              | [in] hBanner          @Banner handle
//              | [in] pfStatus         @Status callback function
//              | [in] pfDisplay        @Display callback function
//              | [in] dwStatusContext  @Status context
//              | [in] dwDisplayContext @Display context
// Return value : N/A
//
CAvSyncLiveCustomDrawWrapper::CAvSyncLiveCustomDrawWrapper(HWND hDlg, HWND hDisplay, HBITMAP hBMP, HBITMAP hBanner, LPSTATUSCALLBACK pfStatus, LPDISPLAYCALLBACK pfDisplay, DWORD dwStatusContext, DWORD dwDisplayContext)
{
	m_hAvSync      = NULL;
	m_hLiveChannel = NULL;

	m_bDrawOnGraph = TRUE;

	m_hBanner = hBanner;
	
	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.hDlg     = hDlg;
	m_tParam.hDisplay = hDisplay;
	m_tParam.hBMP     = hBMP;

	m_tParam.pfStatus  = pfStatus;
	m_tParam.pfDisplay = pfDisplay;
	
	m_tParam.dwStatusContext  = dwStatusContext;
	m_tParam.dwDisplayContext = dwDisplayContext;

	InitialModule();  // Initialize AvSynchronizer and create live channel
}

// Function     : CAvSyncLiveCustomDrawWrapper::~CAvSyncLiveCustomDrawWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CAvSyncLiveCustomDrawWrapper::~CAvSyncLiveCustomDrawWrapper()
{	
	ReleaseModule();  // Delete live channel and release AvSynchronizer
}

// Function     : CAvSyncLiveCustomDrawWrapper::InitialModule
// Description  : Initialize AvSynchronizer and create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::InitialModule()
{	
	SCODE scRet;

	// Set initial flag of AvSynchronizer
	// If "CAPTION_ON_GRAPH" flag is set, the LPDISPLAYCALLBACK will be called every frame. It will cause a worse performance.
	//
	DWORD dwAvSyncInitFlag = CAPTION_ON_GRAPH;

	// STEP 1: Initialize AvSynchronizer
	scRet = AvSynchronizer_InitialEx(&m_hAvSync, m_tParam.pfStatus, m_tParam.pfDisplay, m_tParam.hDlg, AUDIOOUT_FOCUS_NORMAL, 0, AVSYNCHRONIZER_VERSION, dwAvSyncInitFlag, RGB(0, 0, 0));

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InitialEx fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	// STEP 2: Create live channel
	if	(m_hAvSync != NULL)
	{
		scRet = CreateChannel(); 

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::ReleaseModule
// Description  : Delete live channel and release AvSynchronizer
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::ReleaseModule()
{
	SCODE scRet;
	
	if	(m_hAvSync != NULL)
	{
		// STEP 1: Delete live channel
		if	(m_hLiveChannel != NULL)
		{
			scRet = DeleteChannel();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release AvSynchronizer
		scRet = AvSynchronizer_Release(&m_hAvSync);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_Release fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::CreateChannel
// Description  : Create live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::CreateChannel()
{	
	// STEP 1: Set channel options
	TCHANNELOPTION tCHOption;

	memset(&tCHOption, 0, sizeof(tCHOption));

	tCHOption.dwFlags = CH_STATUSCB | CH_DISPLAYCB | CH_STATUSCONTEXT | CH_DISPLAYCONTEXT | CH_ALLBORDER | CH_BITMAP;

	tCHOption.pfStatus  = m_tParam.pfStatus; 
	tCHOption.pfDisplay = m_tParam.pfDisplay;

	tCHOption.dwStatusContext = m_tParam.dwStatusContext;
	tCHOption.dwDisplayContext = m_tParam.dwDisplayContext;
	
	tCHOption.dwTopBorderSize    = 20;
	tCHOption.dwLeftBorderSize   = 0;
	tCHOption.dwBottomBorderSize = 0;
	tCHOption.dwRightBorderSize  = 0;

	tCHOption.hBMP     = m_tParam.hBMP;
	tCHOption.hDisplay = m_tParam.hDisplay;
	
	SCODE scRet;

	// STEP 2: Create live channel
	scRet = AvSynchronizer_CreateChannel(m_hAvSync, &m_hLiveChannel, tCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreateChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}
	
	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::DeleteChannel
// Description  : Delete live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::DeleteChannel()
{
	SCODE scRet;

	// Delete live channel 
	scRet = AvSynchronizer_DeleteChannel(m_hAvSync, &m_hLiveChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::Start
// Description  : Start live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::Start()
{	
	SCODE scRet;

	// Start live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StartChannel(m_hLiveChannel, DRAW_CONNECTING);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StartChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::Stop
// Description  : Stop live channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::Stop()
{
	SCODE scRet;
	
	// Stop live channel
	if	(m_hLiveChannel != NULL)
	{
		scRet = AvSynchronizer_StopChannel(m_hLiveChannel);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StopChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::UpdateChannelMediaType
// Description  : Update channel media type
// Parameter    :
//              | [in] dwMediaType @Media type
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::UpdateChannelMediaType(DWORD dwMediaType)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags    = UPCH_MEDIATYPE;
		tUpdataCHOption.tMediaType = EMEDIATYPE(dwMediaType);
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::UpdateChannelShowBMP
// Description  : Update channel settings to show bitmap
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncLiveCustomDrawWrapper::UpdateChannelShowBMP()
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{	
		TUPDATECHANNELOPTION tUpdataCHOption;

		tUpdataCHOption.dwFlags = UPCH_SHOW_BMP;
	
		scRet = AvSynchronizer_UpdateChannelSettings(m_hLiveChannel, tUpdataCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdateChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncLiveCustomDrawWrapper::InputEncodedMediaFrame
// Description  : Input the encoded media frame into channel
// Parameter    :
//              | [in] pMediaDataPacket @Input media data packet info
// Return value :
//              | S_OK                        @Success
//              | S_FAIL                      @FAIL
//              | AVSYNCHRONIZER_E_QUEUE_FULL @Queue full
//
SCODE CAvSyncLiveCustomDrawWrapper::InputEncodedMediaFrame(TMediaDataPacketInfo *pMediaDataPacket)
{
	SCODE scRet;

	if	(m_hLiveChannel != NULL)
	{		
		scRet = AvSynchronizer_InputEncodedMediaFrame(m_hLiveChannel, pMediaDataPacket);

		if (scRet != S_OK)
		{
			// The return value "AVSYNCHRONIZER_E_QUEUE_FULL" is used to notify the caller that the queue is full.
			// When used in DataBrokerAVCallback function, if the callback function recieves "AVSYNCHRONIZER_E_QUEUE_FULL",
			// it returns "DATABROKER_S_FRAME_NOT_HANDLED" back to DataBroker. Then DataBroker stop putting media data into
			// the queue until it recieves "S_OK."
			//
			if	(scRet == AVSYNCHRONIZER_E_QUEUE_FULL)
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer queue full!\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return scRet;
			}
			else
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputEncodedMediaFrame fail! (ERROR: %X)\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return S_FAIL;
			}
		}
	}

	return S_OK;
}
