#include "stdafx.h"
#include <memory.h>
#include "AvSyncPlaybackGetRawWrapper.h"


// Function     : CAvSyncPlaybackGetRawWrapper::CAvSyncPlaybackGetRawWrapper
// Description  : Constructor
// Parameter    : 
//              | [in] hDlg             @Dialog window handle
//              | [in] hDisplay         @Display window handle
//              | [in] hBMP             @Bitmap handle
//              | [in] pfStatus         @Status callback function
//              | [in] pfDisplay        @Display callback function
//              | [in] dwStatusContext  @Status context
//              | [in] dwDisplayContext @Display context
// Return value : N/A
//
CAvSyncPlaybackGetRawWrapper::CAvSyncPlaybackGetRawWrapper(HWND hDlg, HWND hDisplay, HBITMAP hBMP, LPSTATUSCALLBACK pfStatus, LPDISPLAYCALLBACK pfDisplay, DWORD dwStatusContext, DWORD dwDisplayContext)
{
	m_hAvSync          = NULL;
	m_hPlaybackChannel = NULL;
	
	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.hDlg     = hDlg;
	m_tParam.hDisplay = hDisplay;
	m_tParam.hBMP     = hBMP;
	
	m_tParam.pfStatus  = pfStatus;
	m_tParam.pfDisplay = pfDisplay;
	
	m_tParam.dwStatusContext  = dwStatusContext;
	m_tParam.dwDisplayContext = dwDisplayContext;

	InitialModule();  // Initialize AvSynchronizer and create playback channel
}

// Function     : CAvSyncPlaybackGetRawWrapper::~CAvSyncPlaybackGetRawWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CAvSyncPlaybackGetRawWrapper::~CAvSyncPlaybackGetRawWrapper()
{
	ReleaseModule();  // Delete playback channel and release AvSynchronizer
}

// Function     : CAvSyncPlaybackGetRawWrapper::InitialModule
// Description  : Initialize AvSynchronizer and create playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::InitialModule()
{	
	SCODE scRet;

	// STEP 1: Initialize AvSynchronizer
	scRet = AvSynchronizer_InitialEx(&m_hAvSync, m_tParam.pfStatus, m_tParam.pfDisplay, m_tParam.hDlg, AUDIOOUT_FOCUS_NORMAL, 0, AVSYNCHRONIZER_VERSION, 0, RGB(0, 0, 0));

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InitialEx fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	// STEP 2: Create playback channel
	if	(m_hAvSync != NULL)
	{
		scRet = CreateChannel();

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::ReleaseModule
// Description  : Delete playback channel and release AvSynchronizer
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::ReleaseModule()
{
	SCODE scRet;
	
	if	(m_hAvSync != NULL)
	{
		// STEP 1: Delete playback channel
		if	(m_hPlaybackChannel != NULL)
		{
			scRet = DeleteChannel();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release AvSynchronizer
		scRet = AvSynchronizer_Release(&m_hAvSync);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_Release fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::CreateChannel
// Description  : Create playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::CreateChannel()
{	
	// STEP 1: Set channel options
	TPBCHOPTION tPBCHOption;

	memset(&tPBCHOption, 0, sizeof(tPBCHOption));

	tPBCHOption.dwFlags = PBCH_STATUSCB | PBCH_DISPLAYCB | PBCH_STATUSCONTEXT | PBCH_DISPLAYCONTEXT | PBCH_ALLBORDER | PBCH_BITMAP;

	tPBCHOption.pfStatus  = m_tParam.pfStatus;
	tPBCHOption.pfDisplay = m_tParam.pfDisplay; 

	tPBCHOption.dwStatusContext  = m_tParam.dwStatusContext;
	tPBCHOption.dwDisplayContext = m_tParam.dwDisplayContext;
	
	tPBCHOption.dwTopBorderSize    = 20;
	tPBCHOption.dwLeftBorderSize   = 0;
	tPBCHOption.dwBottomBorderSize = 0;
	tPBCHOption.dwRightBorderSize  = 0;

	tPBCHOption.hBMP     = m_tParam.hBMP;
	tPBCHOption.hDisplay = m_tParam.hDisplay;
	
	SCODE scRet;

	// STEP 2: Create playback channel
	scRet = AvSynchronizer_CreatePlaybackChannel(m_hAvSync, &m_hPlaybackChannel, tPBCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreatePlaybackChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}
	
	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::DeleteChannel
// Description  : Delete playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::DeleteChannel()
{
	SCODE scRet;

	// Delete playback channel 
	scRet = AvSynchronizer_DeleteChannel(m_hAvSync, &m_hPlaybackChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::Play
// Description  : Start playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::Play()
{
	SCODE scRet;
	
	// Start playback channel
	if	(m_hPlaybackChannel != NULL)
	{	
		scRet = AvSynchronizer_StartChannel(m_hPlaybackChannel, 0);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StartChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}
	
	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::Stop
// Description  : Stop playback channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::Stop()
{	
	SCODE scRet;
	
	// Stop playback channel
	if	(m_hPlaybackChannel != NULL)
	{	
		scRet = AvSynchronizer_StopChannel(m_hPlaybackChannel);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_StopChannel fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}

		// Show bitmap and reset play speed to normal
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		DWORD dwNormalPlaySpeed = 1;

		tUpdatePBCHOption.dwFlags = UPCH_SHOW_BMP | UPCH_SETSPEED;
		tUpdatePBCHOption.dwFast  = dwNormalPlaySpeed;
		tUpdatePBCHOption.dwSlow  = dwNormalPlaySpeed;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::Pause
// Description  : Pause or Resume playback channel
// Parameter    :
//              | [in] bPause @TRUE : pause, FALSE : resume
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::Pause(BOOL bPause)
{
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{	
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		tUpdatePBCHOption.dwFlags = UPCH_PAUSE;
		tUpdatePBCHOption.bPause  = bPause;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::Forward
// Description  : Change play speed of playback channel
// Parameter    :
//              | [in] dwFast @Multiple of normal play speed
//              | [in] dwSlow @Divisor of normal play speed
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::Forward(DWORD dwFast, DWORD dwSlow)
{
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{	
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		tUpdatePBCHOption.dwFlags = UPCH_SETSPEED;
		tUpdatePBCHOption.dwFast  = dwFast;
		tUpdatePBCHOption.dwSlow  = dwSlow;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::NextFrame
// Description  : Jump to next frame. The playback channel must be paused in advance!
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::NextFrame()
{
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{	
		TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

		memset(&tUpdatePBCHOption, 0, sizeof(tUpdatePBCHOption));

		tUpdatePBCHOption.dwFlags = PBCH_NEXTFRAME;

		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::UpdatePlaybackChannelMediaType
// Description  : Update channel media type
// Parameter    :
//              | [in] dwMediaType @Media type
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::UpdatePlaybackChannelMediaType(DWORD dwMediaType)
{
	TUPDATEPBCHANNELOPTION tUpdatePBCHOption;

	memset(&tUpdatePBCHOption, 0, sizeof(TUPDATEPBCHANNELOPTION));

	tUpdatePBCHOption.dwFlags = UPCH_MEDIATYPE;
	tUpdatePBCHOption.tMediaType = EMEDIATYPE(dwMediaType);

	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{
		scRet = AvSynchronizer_UpdatePlaybackChannelSettings(m_hPlaybackChannel, tUpdatePBCHOption);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_UpdatePlaybackChannelSettings fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::InputDataPacketToChannel
// Description  : Input the parsed data packet info to playback channel
// Parameter    :
//              | [in] ptPacketInfo @Input media data packet info
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::InputDataPacketToChannel(TMediaDataPacketInfoV3 *ptPacketInfoV3)
{
	// Notice!
	// If the internal buffer of "PLAYBACK CHANNEL" is full, it will block the calling thread by default. 
	// User can call AvSynchronizer_SetChannelOption with SETCH_NOT_BLOCK_WHEN_QUEUE_FULL to enable or disable this function.
	
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{
		TMediaDataPacketInfo *ptPacketInfo = (TMediaDataPacketInfo *)ptPacketInfoV3;  // Cast "ptPacketInfoV3" to (TMediaDataPacketInfo *)

		// Input the parsed data packet info to playback channel
		scRet = AvSynchronizer_InputPlaybackMediaFrame(m_hPlaybackChannel, ptPacketInfo);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputPlaybackMediaFrame fail! (ERROR: %X)\n", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncPlaybackGetRawWrapper::WaitAVQueueEmpty
// Description  : Wait until the audio and video elements in AvSynchronizer's queue are all played 
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::WaitAVQueueEmpty()
{
	SCODE scRet;

	if	(m_hPlaybackChannel != NULL)
	{
		TAVSYNCQUEUESTATUS tAvSyncQueueStatus;

		// Wait until all audio and video elements are played.
		// When all elements are played, the audio buffer queue is unused now, hence the dwAudioQueueElements would be equal to dwAudioQueueSize.
		// On the other hand, the video buffer will keep one unit to redraw the last frame, hence the dwVideoQueueElements = dwVideoQueueSize - 1.
		//
		do{
			Sleep(20);

			memset(&tAvSyncQueueStatus, 0, sizeof(tAvSyncQueueStatus));

			// Get remaining audio and video elements in queue
			scRet = AvSynchronizer_GetRemainingQueueElementCount(m_hPlaybackChannel, &tAvSyncQueueStatus);

			if	(IS_FAIL(scRet))
			{
				sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_GetRemainingQueueElementCount fail! (ERROR: %X)\n", scRet);

				OutputDebugString(m_szErrMsgBuf);

				return S_FAIL;
			}
		
		}while	((tAvSyncQueueStatus.dwAudioQueueElements != tAvSyncQueueStatus.dwAudioQueueSize) || (tAvSyncQueueStatus.dwVideoQueueElements != tAvSyncQueueStatus.dwVideoQueueSize - 1));
	}
	
	return S_OK;
}


// Function     : CAvSyncPlaybackGetRawWrapper::SetDecodeOptions
// Description  : Set decode data format of decoder channel
// Parameter    :
//              | [in] dwFormat             @Decode data format
//              | [in] pfDecodeFrame        @DecodeFrame callback function
//              | [in] dwDecodeFrameContext @DecodeFrame context
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncPlaybackGetRawWrapper::SetDecodeOptions(DWORD dwFormat, LPDECODEFRAMECALLBACK_EX pfDecodeFrameEx, DWORD dwDecodeFrameContext)
{
	m_dwDecodeFormat = dwFormat;
	
	EPIXELFORMAT ePixelFormat = EPIXELFORMAT(dwFormat);

	SCODE scRet;

	scRet = AvSynchronizer_SetChannelOption(m_hPlaybackChannel, SETCH_DECODE_DATA_FORMAT, ePixelFormat, 0);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_SetChannelOption fail! (ERROR: %X)", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}

	// Notice!
	// Here caller should use flag "SETCH_OUTPUT_VIDEO_FRAME" and pass "TRUE" to request the decoded video frame from live channel
	//
	AvSynchronizer_SetChannelOption(m_hPlaybackChannel, SETCH_OUTPUT_VIDEO_FRAME, (DWORD)TRUE, 0);
	
	// Notice!
	// Here caller should use flag "SETCH_DECODE_CBEX", provide a context and a callback function with type "LPDECODEFRAMECALLBACK_EX"
	//
	AvSynchronizer_SetChannelOption(m_hPlaybackChannel, SETCH_DECODE_CBEX, dwDecodeFrameContext, (DWORD)pfDecodeFrameEx);  // Set "SETCH_DECODE_CBEX" to channel

	return S_OK;
}
