#include <Windows.h>
#include "DataBroker.h"
#include "AVSynchronizer.h"


// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext, TMediaDataPacketInfo *ptMediaDataPacket)
{	
	HANDLE hChannel = (HANDLE)dwContext;

	if (hChannel)
	{
		// Input media packet to AVSynchronizer channel
		if (AVSYNCHRONIZER_E_QUEUE_FULL == AvSynchronizer_InputEncodedMediaFrame(hChannel, ptMediaDataPacket))
			return DATABROKER_S_FRAME_NOT_HANDLED;
	}
	
	return S_OK;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{	
	switch (tStatusType)
	{
		case eOnConnectionInfo:
			printf("Get eOnConnectionInfo, start to receive packet\n");
			break;
		case eOnStopped:
			printf("Get eOnStopped, stop to receive packet\n");
			break;
		default:
			break;
	}
	
	return S_OK;
}

// This callback function is used to notify the application before displaying, the calling convention must be stdcall
SCODE __stdcall AvSynchronizerDisplayCallBack(DWORD dwContext, TDISPLAYINFO* ptDisplayInfo)
{
	return S_OK;
}

// This callback function is used to notify the application the status of the AvSynchronizer, the calling convention must be stdcall
SCODE __stdcall AvSynchronizerStatusCallBack(DWORD dwContext, DWORD dwStatusCode, DWORD dwParam1, DWORD dwParam2)
{
	switch	(dwStatusCode)
	{
	case AVSYNCHRONIZER_STATUSCODE_DECODE_ERROR:
		printf("Get AVSYNCHRONIZER_STATUSCODE_DECODE_ERROR, decode error\n");
		break;
	default:
		break;
	}

	return S_OK;
}

int StartStreaming(HANDLE hChannel, HANDLE hConnection)
{
	SCODE scRet;

	scRet = AvSynchronizer_StartChannelBackward(hChannel, DRAW_CONNECTING);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_StartChannel failed with error %X \n", scRet);
		return -1;
	}

	scRet = DataBroker_Connect(hConnection);

	if (IS_FAIL( scRet ))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}

int StopStreaming(HANDLE hChannel, HANDLE hConnection)
{
	SCODE scRet;
	
	scRet = DataBroker_Disconnect(hConnection);

	if (IS_FAIL( scRet ))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}

	scRet = AvSynchronizer_StopChannel(hChannel);	

	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_StopChannel failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

int main(int argc, char *argv[])
{	
	CHAR szClassName[] = "MyWndClass"; 	
	WNDCLASSEX wcex;
	int cxFrame, cyFrame, cyCaption, Width = 352, Height = 288;
	HWND hWnd;
	MSG msg;
	SCODE scRet;
	HANDLE hAVSynchronizer = NULL;
	HANDLE hChannel = NULL;
	TCHANNELOPTION tChannelOption = {0};
	HANDLE hDataBroker = NULL;
	HANDLE hConnection = NULL;
	int iMaxChannel = 32;
	TDataBrokerConnectionOptions tDBConOption = {0};
	// DB streaming sample URL (rewind play). If you eliminate the option "&BACKWARD=yes", the URL is turn to a forward play URL. 
	char szUrl[] = "/Media/Database/Normal?LOCATION=Loc1&STIME=99991231_235959_999&LOCTIME=true&fullspeedbeforejam=yes&BACKWARD=yes&knowgoodbye=true";
	const DWORD dwRTSPPort = 3454; // Darwin server default RTSP port is 3454

	wcex.cbSize         = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= GetModuleHandle(0);
	wcex.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szClassName;
	wcex.hIconSm		= LoadIcon(NULL, IDI_APPLICATION);

	if (!RegisterClassEx(&wcex))
	{
		return -1;
	}

	cxFrame = GetSystemMetrics(SM_CXSIZEFRAME);
	cyFrame = GetSystemMetrics(SM_CYSIZEFRAME);
	cyCaption = GetSystemMetrics(SM_CYCAPTION);
	
	hWnd = CreateWindowEx(0, szClassName, NULL, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, Width + cxFrame * 2, Height + cyCaption + cyFrame * 2, NULL, NULL, GetModuleHandle(NULL), NULL);

	if (!hWnd)
	{
		return -1;
	}

	ShowWindow(hWnd, SW_SHOWDEFAULT);
	UpdateWindow(hWnd);
	
	scRet = AvSynchronizer_InitialEx(&hAVSynchronizer, AvSynchronizerStatusCallBack, AvSynchronizerDisplayCallBack, hWnd, 0, 0, AVSYNCHRONIZER_VERSION, 0, 0);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_InitialEx failed with error %X \n", scRet);
		return -1;
	}

	memset(&tChannelOption, 0, sizeof(TCHANNELOPTION));
	tChannelOption.hDisplay = hWnd;

	scRet = AvSynchronizer_CreateChannel(hAVSynchronizer, &hChannel, tChannelOption);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_CreateChannel failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_Initial(&hDataBroker, iMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, (DWORD)mctALLCODEC, 0, DATABROKER_VERSION);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_CreateConnection(hDataBroker, &hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}
	
	tDBConOption.wHttpPort = 80;
	tDBConOption.dwProtocolType = eptTCP;
	tDBConOption.dwMediaType = emtVideo | emtAudio;
	tDBConOption.pzServerType = "Darwin";
	tDBConOption.pzIPAddr = "127.0.0.1";	
	tDBConOption.pzUID = "admin";
	tDBConOption.pzPWD = "admin";
	tDBConOption.dwAVContext = (DWORD)hChannel;
	tDBConOption.dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort | eConOptAVContext;
	
	scRet = DataBroker_SetConnectionOptions(hConnection, &tDBConOption);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_SetConnectionUrlsExtra(hConnection, szUrl, NULL, NULL, NULL, NULL, NULL);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionUrlsExtra failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_SetConnectionExtraOption(hConnection, eOptRtspCtrlPort, dwRTSPPort, 0);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionExtraOption failed with error %X \n", scRet);
		return -1;
	}

	StartStreaming(hChannel, hConnection);

	while (GetMessage(&msg, NULL, 0, 0) > 0)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	StopStreaming(hChannel, hConnection);

	scRet = DataBroker_DeleteConnection(hDataBroker, &hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_Release(&hDataBroker);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}

	scRet = AvSynchronizer_DeleteChannel(hAVSynchronizer, &hChannel);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_DeleteChannel failed with error %X \n", scRet);
		return -1;
	}

	scRet = AvSynchronizer_Release(&hAVSynchronizer);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_Release failed with error %X \n", scRet);
		return -1;
	}
	
	hConnection = NULL;
	hDataBroker = NULL;
	hChannel = NULL;
	hAVSynchronizer = NULL;

	return (int)msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;

	switch( message )
	{
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		break;
	case WM_CLOSE:
		DestroyWindow(hWnd); 
		break;
	case WM_DESTROY:
		PostQuitMessage( 0 );
		break;

	default:
		return DefWindowProc( hWnd, message, wParam, lParam );
	}

	return 0;
}