#include <Windows.h>
#include "DataBroker.h"
#include "AVSynchronizer.h"

static HANDLE gs_hChannel = NULL;
static HANDLE gs_hConnection = NULL;


// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext, TMediaDataPacketInfo *ptMediaDataPacket)
{	
	HANDLE hChannel = (HANDLE)dwContext;

	if (hChannel)
	{
		// Input media packet to AVSynchronizer channel
		if (AVSYNCHRONIZER_E_QUEUE_FULL == AvSynchronizer_InputEncodedMediaFrame(hChannel, ptMediaDataPacket))
			return DATABROKER_S_FRAME_NOT_HANDLED;
	}
	
	return S_OK;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{	
	switch (tStatusType)
	{
		case eOnConnectionInfo:
			printf("Get eOnConnectionInfo, start to receive packet\n");
			break;
		case eOnStopped:
			printf("Get eOnStopped, stop to receive packet\n");
			break;
		default:
			break;
	}
	
	return S_OK;
}

// This callback function is used to notify the application before displaying, the calling convention must be stdcall
SCODE __stdcall AvSynchronizerDisplayCallBack(DWORD dwContext, TDISPLAYINFO* ptDisplayInfo)
{
	return S_OK;
}

// This callback function is used to notify the application the status of the AvSynchronizer, the calling convention must be stdcall
SCODE __stdcall AvSynchronizerStatusCallBack(DWORD dwContext, DWORD dwStatusCode, DWORD dwParam1, DWORD dwParam2)
{
	switch	(dwStatusCode)
	{
	case AVSYNCHRONIZER_STATUSCODE_DECODE_ERROR:
		printf("Get AVSYNCHRONIZER_STATUSCODE_DECODE_ERROR, decode error\n");
		break;
	default:
		break;
	}

	return S_OK;
}

int StartStreaming(HANDLE hChannel, HANDLE hConnection)
{
	SCODE scRet;

	scRet = AvSynchronizer_StartChannel(hChannel, DRAW_CONNECTING);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_StartChannel failed with error %X \n", scRet);
		return -1;
	}

	scRet = DataBroker_Connect(hConnection);

	if (IS_FAIL( scRet ))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}

int StopStreaming(HANDLE hChannel, HANDLE hConnection)
{
	SCODE scRet;
	
	scRet = DataBroker_Disconnect(hConnection);

	if (IS_FAIL( scRet ))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}

	scRet = AvSynchronizer_StopChannel(hChannel);	

	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_StopChannel failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}

static SCODE ForwardPlayback(DWORD dwFast, DWORD dwSlow)
{
	SCODE scRet = S_OK;
	TUPDATEPBCHANNELOPTION tUpdateChannelOption = {0};
	
	scRet = DataBroker_SetConnectionExtraOption(gs_hConnection, eOptRTSPForward, dwFast, dwSlow);			
	
	if (IS_FAIL( scRet ))
	{
		printf("DataBroker_SetConnectionExtraOption failed with error %X \n", scRet);
		return scRet;
	}			
	
	tUpdateChannelOption.dwFlags = UPCH_SETSPEED;
	tUpdateChannelOption.dwFast = dwFast;
	tUpdateChannelOption.dwSlow = dwSlow;

	scRet = AvSynchronizer_UpdatePlaybackChannelSettings(gs_hChannel, tUpdateChannelOption);

	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_UpdatePlaybackChannelSettings failed with error %X \n", scRet);
		return scRet;
	}

	return scRet;
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

int main(int argc, char *argv[])
{	
	CHAR szClassName[] = "MyWndClass"; 	
	WNDCLASSEX wcex;
	int cxFrame, cyFrame, cyCaption, Width = 352, Height = 288;
	HWND hWnd;
	MSG msg;
	SCODE scRet;
	HANDLE hAVSynchronizer = NULL;
	TPBCHOPTION tChannelOption = {0};
	HANDLE hDataBroker = NULL;
	int iMaxChannel = 32;
	TDataBrokerConnectionOptions tDBConOption = {0};
	char szUrl[] = "/Media/Database/Normal?LOCATION=Loc1&STIME=20150831_085000&ETIME=20150828_100000&LOCTIME=true"; // DB streaming sample URL
	const DWORD dwRTSPPort = 3454; // Darwin server default RTSP port is 3454

	wcex.cbSize         = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= GetModuleHandle(0);
	wcex.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szClassName;
	wcex.hIconSm		= LoadIcon(NULL, IDI_APPLICATION);

	if (!RegisterClassEx(&wcex))
	{
		return -1;
	}

	cxFrame = GetSystemMetrics(SM_CXSIZEFRAME);
	cyFrame = GetSystemMetrics(SM_CYSIZEFRAME);
	cyCaption = GetSystemMetrics(SM_CYCAPTION);
	
	hWnd = CreateWindowEx(0, szClassName, NULL, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, Width + cxFrame * 2, Height + cyCaption + cyFrame * 2, NULL, NULL, GetModuleHandle(NULL), NULL);

	if (!hWnd)
	{
		return -1;
	}

	ShowWindow(hWnd, SW_SHOWDEFAULT);
	UpdateWindow(hWnd);
	
	scRet = AvSynchronizer_InitialEx(&hAVSynchronizer, AvSynchronizerStatusCallBack, AvSynchronizerDisplayCallBack, hWnd, 0, 0, AVSYNCHRONIZER_VERSION, 0, 0);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_InitialEx failed with error %X \n", scRet);
		return -1;
	}

	memset(&tChannelOption, 0, sizeof(TPBCHOPTION));
	tChannelOption.hDisplay = hWnd;

	// Playback channel
	scRet = AvSynchronizer_CreatePlaybackChannel(hAVSynchronizer, &gs_hChannel, tChannelOption);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_CreatePlaybackChannel failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_Initial(&hDataBroker, iMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, (DWORD)mctALLCODEC, 0, DATABROKER_VERSION);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_CreateConnection(hDataBroker, &gs_hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}
	
	tDBConOption.wHttpPort = 80;
	tDBConOption.dwProtocolType = eptTCP;
	tDBConOption.dwMediaType = emtVideo | emtAudio;
	tDBConOption.pzServerType = "Darwin";
	tDBConOption.pzIPAddr = "127.0.0.1";	
	tDBConOption.pzUID = "admin";
	tDBConOption.pzPWD = "admin";
	tDBConOption.dwAVContext = (DWORD)gs_hChannel;
	tDBConOption.dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort | eConOptAVContext;
	
	scRet = DataBroker_SetConnectionOptions(gs_hConnection, &tDBConOption);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_SetConnectionUrlsExtra(gs_hConnection, szUrl, NULL, NULL, NULL, NULL, NULL);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionUrlsExtra failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_SetConnectionExtraOption(gs_hConnection, eOptRtspCtrlPort, dwRTSPPort, 0);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionExtraOption failed with error %X \n", scRet);
		return -1;
	}

	StartStreaming(gs_hChannel, gs_hConnection);

	while (GetMessage(&msg, NULL, 0, 0) > 0)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	StopStreaming(gs_hChannel, gs_hConnection);

	scRet = DataBroker_DeleteConnection(hDataBroker, &gs_hConnection);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}
	
	scRet = DataBroker_Release(&hDataBroker);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}

	scRet = AvSynchronizer_DeleteChannel(hAVSynchronizer, &gs_hChannel);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_DeleteChannel failed with error %X \n", scRet);
		return -1;
	}

	scRet = AvSynchronizer_Release(&hAVSynchronizer);
	
	if (IS_FAIL( scRet ))
	{
		printf("AvSynchronizer_Release failed with error %X \n", scRet);
		return -1;
	}
	
	gs_hConnection = NULL;
	hDataBroker = NULL;
	gs_hChannel = NULL;
	hAVSynchronizer = NULL;

	return (int)msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;

	switch( message )
	{
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		break;
	case WM_CLOSE:
		DestroyWindow(hWnd); 
		break;
	case WM_DESTROY:
		PostQuitMessage( 0 );
		break;
	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_OEM_PLUS:
			{
				// 4X forward playback				
				DWORD dwFast = 4, dwSlow = 1;				
				ForwardPlayback(dwFast, dwSlow);
			}
			break;
		case VK_OEM_MINUS:
			{
				// 0.25X forward playback				
				DWORD dwFast = 1, dwSlow = 4;
				ForwardPlayback(dwFast, dwSlow);
			}
			break;
		case 0x30: // '0'
			{
				// 1X forward playback				
				DWORD dwFast = 1, dwSlow = 1;
				ForwardPlayback(dwFast, dwSlow);
			}
			break;
		}
		break;

	default:
		return DefWindowProc( hWnd, message, wParam, lParam );
	}

	return 0;
}