// GetPresentContent.cpp : Defines the entry point for the application.
//

#include "GetPresentContent.h"

#define MAX_LOADSTRING 100

CMPUT                  *g_pMPUT = NULL;
TFISHEYEVIEWER         g_tViewer;
TFISHEYESNAPSHOT       g_tFisheyeSnapshot;
TFISHEYECURRENTDISPLAY g_tFisheyeDisplay;

// Fisheye Stream Function Declaration
// ----------------------------------------------------------------------------------------------------
void connect();
void disconnect();
// ----------------------------------------------------------------------------------------------------

// Fisheye Viewer Function Declaration
// ----------------------------------------------------------------------------------------------------
void updateViewer(TFISHEYEVIEWER *ptViewer);
// ----------------------------------------------------------------------------------------------------

// Fisheye Get Present Content Function Declaration
// ----------------------------------------------------------------------------------------------------
void getCurrentSnapshot(TFISHEYESNAPSHOT *ptFisheyeSnapshot, BOOL bOrginalSize);
void getCurrentDisplay(HDC hDC, TFISHEYECURRENTDISPLAY *ptFisheyeCurrentDisplay);
// ----------------------------------------------------------------------------------------------------

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_GETPRESENTCONTENT, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow))
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDC_GETPRESENTCONTENT));

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (int) msg.wParam;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_GETPRESENTCONTENT));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_GETPRESENTCONTENT);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HINSTANCE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   HWND hWnd;

   hInst = hInstance; // Store instance handle in our global variable

   hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, 800, 600, NULL, NULL, hInstance, NULL);

   if (!hWnd)
   {
      return FALSE;
   }

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

   g_tViewer.hViewerWindow = hWnd;
   g_tFisheyeSnapshot.hViewerWindow = hWnd;
   
   g_pMPUT = new CMPUT(hWnd);
   if (!g_pMPUT)
   {
	   return FALSE;
   }

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	//PAINTSTRUCT ps;
	//HDC hdc;

	switch (message)
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		// Parse the menu selections:
		switch (wmId)
		{
		case IDM_ABOUT:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUTBOX), hWnd, About);
			break;
		case IDM_EXIT:
			if (g_pMPUT)
			{
				delete g_pMPUT;
				g_pMPUT = NULL;
			}
			DestroyWindow(hWnd);
			break;

		// Fisheye Stream Operation
		// ----------------------------------------------------------------------------------------------------
		case ID_FISHEYE_STREAM_CONNECT:
			connect();
			break;
		case ID_FISHEYE_STREAM_DISCONNECT:
			disconnect();
			break;
		// ----------------------------------------------------------------------------------------------------

		// Fisheye Viewer Function Operation: Select present mode
		// ----------------------------------------------------------------------------------------------------
		case ID_FISHEYE_PM_SINGLE_SOURCE_STRETCH:
			g_tViewer.dwPresentMode = FEPRESENTMODE_SINGLE_SOURCE_STRETCH;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_PRESENTMODE;
			updateViewer(&g_tViewer);
			break;
		case ID_FISHEYE_PM_SINGLE_PARTIALCORRECTION:
			g_tViewer.dwPresentMode = FEPRESENTMODE_SINGLE_PERSPECT_VIEW;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_PRESENTMODE;
			updateViewer(&g_tViewer);
			break;
		case ID_FISHEYE_PM_QUAD_UNCALIBRATE:
			g_tViewer.dwPresentMode = FEPRESENTMODE_QUAD_WITH_SOURCE_CONSTRAINPROP;
			g_tViewer.dwUpdateFlags = FEUPDATEVIEWER_PRESENTMODE;
			updateViewer(&g_tViewer);
			break;
		// ----------------------------------------------------------------------------------------------------

		// Fisheye Get Present Content Function Operation
		// ----------------------------------------------------------------------------------------------------
		case ID_FISHEYE_SNAPSHOT_CURRENTVIEW:
			getCurrentSnapshot(&g_tFisheyeSnapshot, FALSE);
			break;
		case ID_FISHEYE_SNAPSHOT_ORIGINALSIZE:
			getCurrentSnapshot(&g_tFisheyeSnapshot, TRUE);
			break;
		case ID_FISHEYE_DISPLAY:
			{
				RECT windowRoi;
				GetWindowRect(hWnd, &windowRoi);
				
				HWND hDisplayWindow = CreateWindow("STATIC", NULL, WS_OVERLAPPEDWINDOW, windowRoi.right, windowRoi.top, (windowRoi.right - windowRoi.left), (windowRoi.bottom - windowRoi.top), NULL, NULL, NULL, NULL);
				ShowWindow(hDisplayWindow, SW_NORMAL);
				UpdateWindow(hDisplayWindow);
				
				HDC hDisplayDC = GetDC(hDisplayWindow);
				
				g_tFisheyeDisplay.hViewerWindow = hWnd;
				getCurrentDisplay(hDisplayDC, &g_tFisheyeDisplay);

				Sleep(5000);				
				ReleaseDC(hDisplayWindow, hDisplayDC);
				DestroyWindow(hDisplayWindow);
			}
		// ----------------------------------------------------------------------------------------------------

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_PAINT:
		//hdc = BeginPaint(hWnd, &ps);
		// TODO: Add any drawing code here...
		//EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
INT_PTR CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(lParam);
	switch (message)
	{
	case WM_INITDIALOG:
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}

// Fisheye Stream Function Definition
// ----------------------------------------------------------------------------------------------------
void connect()
{
	if (g_pMPUT)
	{
		BOOL bSupportFisheye;
		bSupportFisheye = g_pMPUT->FisheyeCheckSupport();
		if (!bSupportFisheye)
		{
			MessageBox(g_tViewer.hViewerWindow, "Not support fisheye render!", NULL, (MB_OK | MB_ICONWARNING));
			return;
		}

		g_pMPUT->FisheyeEnableRender();

		g_pMPUT->Connect();
	}
}

void disconnect()
{
	if (g_pMPUT)
	{
		g_pMPUT->Disconnect();
	}
}
// ----------------------------------------------------------------------------------------------------

// Fisheye Viewer Function Definition
// ----------------------------------------------------------------------------------------------------
void updateViewer(TFISHEYEVIEWER *ptViewer)
{
	if (g_pMPUT)
	{
		g_pMPUT->FisheyeUpdateViewer(ptViewer);
	}
}
// ----------------------------------------------------------------------------------------------------

// Fisheye Get Present Content Function Definition
// ----------------------------------------------------------------------------------------------------
void getCurrentSnapshot(TFISHEYESNAPSHOT *ptFisheyeSnapshot, BOOL bOrginalSize)
{
	if (g_pMPUT)
	{
		g_pMPUT->FisheyeGetCurrentViewerSnapshot(ptFisheyeSnapshot, "C:\\", bOrginalSize);
	}
}

void getCurrentDisplay(HDC hDC, TFISHEYECURRENTDISPLAY *ptFisheyeCurrentDisplay)
{
	if (g_pMPUT)
	{
		g_pMPUT->FisheyeGetCurrentViewerDisplay(hDC, ptFisheyeCurrentDisplay);
	}
}
// ----------------------------------------------------------------------------------------------------