#include "MPUT.h"
#include "DataBroker.h"
#include "AvSynchronizer.h"

// Helper Macros
// ----------------------------------------------------------------------------------------------------
#ifdef _DEBUG
#ifndef MP_CALL
#define MP_CALL( call ) {                                                                             \
	SCODE sc = (call);                                                                                \
	if ( IS_FAIL(sc) ) {                                                                              \
		char szdbg[MAX_PATH] = {0};                                                                   \
		sprintf_s(szdbg, sizeof(szdbg), "%s(%d): error : 0x%x\n", __FILE__, __LINE__, sc);            \
		OutputDebugString(szdbg);                                                                     \
		return sc;                                                                                    \
	}}
#endif
#else
#ifndef MP_CALL
#define MP_CALL( call ) { SCODE sc = (call); if ( IS_FAIL(sc) ) { return sc; } }                             
#endif
#endif
// ----------------------------------------------------------------------------------------------------

// Callback Functions Declaration
// ----------------------------------------------------------------------------------------------------
#pragma region Callback Functions Declaration
// ----------------------------------------------------------------------------------------------------
static SCODE __stdcall DBStatusCB(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2);
static SCODE __stdcall DBAVCB(DWORD dwContext, TMediaDataPacketInfo *pMediaDataPacket);                                
static SCODE __stdcall ASStatusCB(DWORD dwContext, DWORD dwStatusCode, DWORD dwParam1, DWORD dwParam2);           
static SCODE __stdcall ASDisplayCB(DWORD dwContext, TDISPLAYINFO* ptDisplayInfo);   
// ----------------------------------------------------------------------------------------------------
#pragma endregion
// ----------------------------------------------------------------------------------------------------

// Helper Class
// ----------------------------------------------------------------------------------------------------
#pragma region Helper Class
// ----------------------------------------------------------------------------------------------------
CMPUT::CMPUT(HWND hShowWnd) :
m_hShowWnd(hShowWnd),
m_hASChannel(NULL),
m_bConnected(FALSE),
m_hAS(NULL),
m_hDB(NULL)
{
	internalInitial();
}

void CMPUT::internalInitial()
{	
	internalInitialDB();
	internalInitialAS();
}

SCODE CMPUT::internalInitialDB()
{	
	DWORD dwCodecs = (mctJPEG | mctH263 | mctMP4V | mctG7221 | mctG729A | mctAAC | mctGAMR | mctSAMR);
	MP_CALL( DataBroker_Initial(&m_hDB, 1, DBStatusCB, DBAVCB, dwCodecs, 0, DATABROKER_VERSION) );

	MP_CALL( DataBroker_CreateConnection(m_hDB, &m_hDBConnect) );

	return S_OK;
}

SCODE CMPUT::internalInitialAS()
{	
	DWORD dwInitFlag = USE_DIRECTDRAW_ONLY;
	MP_CALL( AvSynchronizer_InitialEx(&m_hAS, ASStatusCB, ASDisplayCB, m_hShowWnd, AUDIOOUT_FOCUS_NORMAL, 0, AVSYNCHRONIZER_VERSION, dwInitFlag, RGB(0, 0, 0)) );

	TCHANNELOPTION tChannelOption;
	memset(&tChannelOption, 0, sizeof(TCHANNELOPTION));
	
	tChannelOption.dwFlags          = (CH_STATUSCB | CH_DISPLAYCB | CH_STATUSCONTEXT | CH_DISPLAYCONTEXT | CH_ALLBORDER);
	tChannelOption.pfStatus         = ASStatusCB; 
	tChannelOption.pfDisplay        = ASDisplayCB;
	tChannelOption.dwStatusContext  = (DWORD)this;
	tChannelOption.dwDisplayContext = (DWORD)this;
	tChannelOption.dwTopBorderSize  = 20;
	tChannelOption.hDisplay         = m_hShowWnd;

	MP_CALL( AvSynchronizer_CreateChannel(m_hAS, &m_hASChannel, tChannelOption) );

	return S_OK;
}

CMPUT::~CMPUT()
{
	internalRelease();
}

void CMPUT::internalRelease()
{
	internalReleaseAS();
	internalReleaseDB();
}

SCODE CMPUT::internalReleaseAS()
{
	MP_CALL( AvSynchronizer_StopChannel(m_hASChannel) );	
	MP_CALL( AvSynchronizer_DeleteChannel(m_hAS, &m_hASChannel) );
	MP_CALL( AvSynchronizer_Release(&m_hAS) );

	return S_OK;
}

SCODE CMPUT::internalReleaseDB()
{	
	MP_CALL( DataBroker_Disconnect(m_hDBConnect) );

	DWORD dwWaitTime = 30;
	while (m_bConnected)
	{
		Sleep(dwWaitTime);
	}
	
	MP_CALL( DataBroker_DeleteConnection(m_hDB, &m_hDBConnect) );
	MP_CALL( DataBroker_Release(&m_hDB) );

	return S_OK;
}

SCODE CMPUT::Connect()
{
	MP_CALL( AvSynchronizer_StartChannel(m_hASChannel, DRAW_CONNECTING) );	

	TDataBrokerConnectionOptions tDBConnOptions;
	memset(&tDBConnOptions, 0, sizeof(TDataBrokerConnectionOptions));
	
	tDBConnOptions.dwFlags         = (eConOptProtocolAndMediaType | eConOptHttpPort | eConOptStatusCallback | eConOptAVCallback | eConOptStatusContext | eConOptAVContext);
	tDBConnOptions.pzIPAddr        = "172.16.81.73";
	tDBConnOptions.pzServerType    = "auto";
	tDBConnOptions.pzUID           = "root";
	tDBConnOptions.pzPWD           = "1234";
	tDBConnOptions.wHttpPort       = 80;
	tDBConnOptions.dwProtocolType  = eptTCP;
	tDBConnOptions.dwMediaType     = (emtVideo | emtAudio);
	tDBConnOptions.pfStatus        = DBStatusCB;
	tDBConnOptions.pfAV            = DBAVCB;
	tDBConnOptions.dwStatusContext = (DWORD)this;
	tDBConnOptions.dwAVContext     = (DWORD)this;

	MP_CALL( DataBroker_SetConnectionOptions(m_hDBConnect,  &tDBConnOptions) );

	DWORD dwOption = eOptStreamIndex;
	DWORD dwStream = 0;
	MP_CALL( DataBroker_SetConnectionExtraOption(m_hDBConnect, dwOption, dwStream, 0) );	

	MP_CALL( DataBroker_Connect(m_hDBConnect) );

	return S_OK;
}

SCODE CMPUT::Disconnect()
{
	MP_CALL( DataBroker_Disconnect(m_hDBConnect) );
	
	MP_CALL( AvSynchronizer_StopChannel(m_hASChannel) );

	return S_OK;
}

BOOL CMPUT::FisheyeCheckSupport()
{
	BOOL bSupportFisheye;
	AvSynchronizer_FisheyeCheckSupport(&bSupportFisheye);

	return bSupportFisheye;
}

SCODE CMPUT::FisheyeEnableRender()
{	
	MP_CALL( AvSynchronizer_FisheyeEnableRenderer(m_hASChannel) );

	return S_OK;
}

SCODE CMPUT::FisheyeRegisterViewer(TFISHEYEVIEWER *ptFisheyeViewer)
{
	MP_CALL( AvSynchronizer_FisheyeRegisterViewer(m_hASChannel, ptFisheyeViewer) );

	return S_OK;
}

SCODE CMPUT::FisheyeUpdateViewer(TFISHEYEVIEWER *ptFisheyeViewer)
{	
	MP_CALL( AvSynchronizer_FisheyeUpdateViewer(m_hASChannel, ptFisheyeViewer) );

	return S_OK;
}

SCODE CMPUT::FisheyeRemoveViewer(TFISHEYEVIEWER *ptFisheyeViewer)
{	
	MP_CALL( AvSynchronizer_FisheyeRemoveViewer(m_hASChannel, ptFisheyeViewer) );

	return S_OK;
}

SCODE CMPUT::FisheyeGetCurrentViewerSnapshot(TFISHEYESNAPSHOT *ptFisheyeSnapshot, LPCSTR pszDirectory, BOOL bOrginalSize)
{	
	DWORD dwSnapshotFlags = ((PF_JPEG << 16) | SPECIFY_FMT);
	
	if (bOrginalSize)
	{
		dwSnapshotFlags |= ORIGINAL_SIZE;
	}
	else
	{
		dwSnapshotFlags |= VISIBLE_SIZE;
	}	
	
	MP_CALL( AvSynchronizer_FisheyeGetCurrentViewerSnapShot(m_hASChannel, ptFisheyeSnapshot, dwSnapshotFlags) ); 

	CHAR szPath[MAX_PATH] = "\0";
	sprintf_s(szPath, sizeof(szPath), "%s\\snapshot_viewer(0x%x).jpg", pszDirectory, ptFisheyeSnapshot->hViewerWindow);

	FILE *pf;
	fopen_s(&pf, szPath, "wb");
	fwrite(ptFisheyeSnapshot->tSnapshot.pDataStart, 1, ptFisheyeSnapshot->tSnapshot.dwdataSize, pf);
	fclose(pf);

	MP_CALL( AvSynchronizer_FreePicture(&ptFisheyeSnapshot->tSnapshot) );

	return S_OK;
}

SCODE CMPUT::FisheyeGetCurrentViewerDisplay(HDC hDC, TFISHEYECURRENTDISPLAY *ptFisheyeCurrentDisplay)
{
	MP_CALL( AvSynchronizer_FisheyeGetCurrentViewerDisplay(m_hASChannel, hDC, ptFisheyeCurrentDisplay) );

	return S_OK;
}

SCODE CMPUT::FisheyeGetViewerPresentParameters(TFISHEYEPRESENTPARAMS *ptFisheyePresentParams)
{
	MP_CALL( AvSynchronizer_FisheyeGetViewerPresentParameters(m_hASChannel, ptFisheyePresentParams) );

	return S_OK;
}

SCODE CMPUT::FisheyeSetViewerPresentParameters(TFISHEYEPRESENTPARAMS *ptFisheyePresentParams)
{
	MP_CALL( AvSynchronizer_FisheyeSetViewerPresentParameters(m_hASChannel, ptFisheyePresentParams) );

	return S_OK;
}

SCODE CMPUT::FisheyeUpdateRotateOptions(TFISHEYEROTATEOPTIONS *ptRotateOptions)
{
	MP_CALL( AvSynchronizer_FisheyeUpdateRotateOptions(m_hASChannel, ptRotateOptions) );

	return S_OK;
}
// ----------------------------------------------------------------------------------------------------
#pragma endregion
// ----------------------------------------------------------------------------------------------------

// Callback Functions Definition
// ----------------------------------------------------------------------------------------------------
#pragma region Callback Functions Definition
// ----------------------------------------------------------------------------------------------------
static SCODE __stdcall ASStatusCB(DWORD dwContext, DWORD dwStatusCode, DWORD dwParam1, DWORD dwParam2)
{
	CMPUT *pMPUTObj = (CMPUT*)dwContext;
	
	UNREFERENCED_PARAMETER(dwParam2);
	
	switch (dwStatusCode)
	{
	// Callback fisheye camera mount type
	case AVSYNCHRONIZER_STATUSCODE_FISHEYE_MOUNT_TYPE:
		{					
			EFISHEYEVIEWERANGLEFLAGS eViewAngle;
			
			EAVSYNCFISHEYEMOUNTTYPE eType = EAVSYNCFISHEYEMOUNTTYPE(dwParam1);
			
			switch (eType)
			{
			case AVSVYNCRHONIZER_FISHEYE_WALL:
				eViewAngle = FEVIEWERANGLE_180;
				break;
			case AVSVYNCRHONIZER_FISHEYE_CEILING:
				eViewAngle = FEVIEWERANGLE_360;
				break;
			case AVSVYNCRHONIZER_FISHEYE_FLOOR:
				eViewAngle = FEVIEWERANGLE_360I;
				break;
			default:
				eViewAngle = FEVIEWERANGLE_180;
				break;
			}

			// User can update the viewer's setting here
			TFISHEYEVIEWER tViewer = {0};
			tViewer.hViewerWindow = pMPUTObj->m_hShowWnd;
			tViewer.dwViewerAngle = eViewAngle;
			tViewer.dwUpdateFlags = FEUPDATEVIEWER_VIEWERANGLE;
			AvSynchronizer_FisheyeUpdateViewer(pMPUTObj->m_hASChannel, &tViewer);

			break;
		}
	default:
		break;
	}	
	
	return S_OK;
}

static SCODE __stdcall ASDisplayCB(DWORD dwContext, TDISPLAYINFO* ptDisplayInfo)
{
	HDC hDisplayDC = ptDisplayInfo->hDC;

	UNREFERENCED_PARAMETER(dwContext);

	SetBkMode(hDisplayDC, TRANSPARENT);
	SetTextColor(hDisplayDC, RGB(255, 255, 255));

	if	((ptDisplayInfo->dwFlag & DIF_CONNECTING) == 0)
	{	
		if	((ptDisplayInfo->dwFlag & DIF_RESUMING) == 0)
		{		
			struct tm *ptmTimeStamp = (struct tm *)ptDisplayInfo->DrawTime;	

			if (ptmTimeStamp)
			{
				char buf[MAX_PATH] = "\0";

				sprintf_s(buf, sizeof(buf), "%d/%02d/%02d %02d:%02d:%02d", ptmTimeStamp->tm_year + 1900,
																		   ptmTimeStamp->tm_mon + 1, 
																		   ptmTimeStamp->tm_mday,
																		   ptmTimeStamp->tm_hour, 
																		   ptmTimeStamp->tm_min, 
																		   ptmTimeStamp->tm_sec);

				SIZE bufSize;
				GetTextExtentPoint32(hDisplayDC, buf, strlen(buf), &bufSize);			
				TextOut(hDisplayDC, ptDisplayInfo->dwWidth - bufSize.cx - 5, ptDisplayInfo->ptOrigin.y, buf, strlen(buf));			
			}
		}
	}
	else
	{
		char buf[MAX_PATH] = "\0";
		sprintf_s(buf, sizeof(buf), "Connecting...");
		
		SIZE bufSize;
		GetTextExtentPoint32(hDisplayDC, buf, strlen(buf), &bufSize);		
		TextOut(hDisplayDC, ptDisplayInfo->dwWidth - bufSize.cx - 5, ptDisplayInfo->ptOrigin.y, buf, strlen(buf));
	}

	return S_OK;
}

static SCODE __stdcall DBStatusCB(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{
	CMPUT *pMPUTObj = (CMPUT*)dwContext;

	UNREFERENCED_PARAMETER(pvParam2);
	
	switch (tStatusType)
	{
	case eOnConnectionInfo:
		{
			pMPUTObj->m_bConnected = TRUE;
			OutputDebugString("Connect to remote server successfully.\n");

			TUPDATECHANNELOPTION tUpdataChannelOption;
			memset(&tUpdataChannelOption, 0, sizeof(TUPDATECHANNELOPTION));			
			
			tUpdataChannelOption.dwFlags = UPCH_MEDIATYPE;
			
			TDataBrokerConnInfo *ptDBConnInfo = (TDataBrokerConnInfo*)pvParam1;			
			switch (ptDBConnInfo->dwMediaType)
			{
			case (emtAudio|emtVideo):
				{
					tUpdataChannelOption.tMediaType = AVSYNCHRONIZER_MEDIATYPE_AUDIO_VIDEO;
					break;
				}
			case emtVideo:
				{
					tUpdataChannelOption.tMediaType = AVSYNCHRONIZER_MEDIATYPE_VIDEO_ONLY;
					break;
				}
			case emtAudio:
				{
					tUpdataChannelOption.tMediaType = AVSYNCHRONIZER_MEDIATYPE_AUDIO_ONLY;
					break;
				}
			}

			if (pMPUTObj->m_hASChannel)
			{
				MP_CALL( AvSynchronizer_UpdateChannelSettings(pMPUTObj->m_hASChannel, tUpdataChannelOption) );
			}
			
			break;
		}
	case eOnStopped:
		{
			pMPUTObj->m_bConnected = FALSE;
			OutputDebugString("Disconnect from remote server successfully.\n");
			break;
		}
	case eOnStartMediaChannel:
		{
			OutputDebugString("Begin to receive media stream from remote server.\n");
			break;
		}
	case eOnAuthFailed:
		{
			OutputDebugString("Authentication failed!\n");
			break;
		}
	case eOnServiceUnavailable:
		{
			OutputDebugString("Connections to remote server already exceed the upper limit!\n");
			break;
		}
	default:
		break;
	}

	return S_OK;
}

static SCODE __stdcall DBAVCB(DWORD dwContext, TMediaDataPacketInfo *pMediaDataPacket)
{
	CMPUT *pMPUTObj = (CMPUT*)dwContext;

	TMediaDataPacketInfoV3 *pMDPInfoV3 = (TMediaDataPacketInfoV3*)pMediaDataPacket;
	if (pMDPInfoV3->tIfEx.tInfo.dwStreamType < mctG7221)
	{
		// Media packet of video
	}
	else
	{
		// Media packet of audio
	}

	if (pMPUTObj->m_hASChannel)
	{
		SCODE scRet;

		scRet = AvSynchronizer_InputEncodedMediaFrame(pMPUTObj->m_hASChannel, pMediaDataPacket);		
		if (scRet == AVSYNCHRONIZER_E_QUEUE_FULL)
		{
			OutputDebugString("Call to AvSynchronizer_InputEncodedMediaFrame fail with queue full!\n");
			return DATABROKER_S_FRAME_NOT_HANDLED;
		}
	}

	return S_OK;
}
// ----------------------------------------------------------------------------------------------------
#pragma endregion
// ----------------------------------------------------------------------------------------------------