// Control.cpp : Defines the entry point for the console application.
//
#ifdef _WIN32
#include <winsock.h>
#endif // _WIN32

#include "stdlib.h"
#include "stdio.h"
#include "DRMControl.h"

DWORD __stdcall DRMControlCallback(HANDLE hParentObject, EDRMControl_CallbackType eCallbackType, void* pvCallbackData)
{
	UNREFERENCED_PARAMETER(hParentObject);
	tagTDRMServiceInfo* ptDRMServiceInfo;
	HANDLE hDRMControl;

	switch(eCallbackType)
	{
	case DRMControl_Callback_DiscoveryResult :
		ptDRMServiceInfo= (tagTDRMServiceInfo*)pvCallbackData;
		printf("IP address: %-15s ",ptDRMServiceInfo->acIP);
		printf("Server Type: %s ",ptDRMServiceInfo->acMachineType);
		printf("CameraName:%s\n",ptDRMServiceInfo->acCameraName);
		
		printf("MAC address:");
		for(int i = 0; i < 6; i++)
		{
			printf("%02x",ptDRMServiceInfo->abyMac[i]);
		}
		printf(" HTTP port: %-4d ",ptDRMServiceInfo->sHTTPPort);
		printf("FTP port: %-4d ",ptDRMServiceInfo->sFTPPort);
		printf("Language: %s\n\n",ptDRMServiceInfo->acLanguage);
		break;

	case DRMControl_Callback_Stopped:
		hDRMControl = (HANDLE) pvCallbackData;
		printf("DRM thread stopped\n");
		break;

	case DRMControl_Callback_Error :
		printf("broadcast error!!\n");
		break;

	default:
		break;
	}
	return 0;
}


int main(void)
{
	HANDLE hDRMControl;
	TDRMControlCreateOptions DRMOption;

	//	STEP 1: Startup windows socket module
	WSADATA wsaData;
	if (WSAStartup(0x202, &wsaData) == SOCKET_ERROR)
	{
		fprintf(stderr,"WSAStartup error %d\n",WSAGetLastError());
		return -1;
	}
	else
	{
		printf("WSAStartup successed.\n");
	}

	//	STEP 2: Create DRMControl 
	DRMOption.usDRMControlPort = 9000;
	DRMControl_Create(&hDRMControl, &DRMOption);

	//	STEP 3: Set callback function
	DRMControl_SetCallback(hDRMControl, (DWORD) NULL, DRMControlCallback);

	//	STEP 4: Start the DRMControl
	DRMControl_Start(hDRMControl);

	//	STEP 5: Start to discover devices in the LAN.
	printf("Start to discover the devices in the LAN\n");
	DRMControl_Discovery(hDRMControl);    // discover all device in the LAN      
	Sleep(10000);

	printf("Discover device by MAC\n");
	DRMControl_DiscoveryByMAC(hDRMControl,"0002D100089C"); // discover this MAC only 
	Sleep(10000);

	//	STEP 7:	Stop the DRMControl
	DRMControl_Stop(hDRMControl);

	//	STEP 8: Close the DRMControl
	DRMControl_Close(&hDRMControl);

	//	STEP 9: Cleanup the windows socket module
	WSACleanup();

	return 0;
}

