#include <conio.h>
#include "tchar.h"
#include "DataBroker.h"

#define MAX_IP_LENGTH		256
#define MAX_USERNAME_LENGTH 256
#define MAX_PASSWORD_LENGTH 256



// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext,TMediaDataPacketInfo *ptMediaDataPacket)
{
   // Show video I frame time stamp for receive packet
   if (ptMediaDataPacket->tFrameType == MEDIADB_FRAME_INTRA && ptMediaDataPacket->dwStreamType < mctG7221)
   { 
       printf("Video I frame time stamp : %u %u \n", ptMediaDataPacket->dwFirstUnitSecond, ptMediaDataPacket->dwFirstUnitMilliSecond);
   }
  
   return S_OK;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType tStatusType, PVOID pvParam1, PVOID pvParam2)
{
	switch (tStatusType)
	{
		case eOnConnectionInfo:
		{
			printf("Get eOnConnectionInfo, start to receive packet\n");
		}
		break;

		case eOnStopped:
		{
			printf("Get eOnStopped, stop to receive packet\n");
		}
		break;

		default:
		{
		}	
		break;
	}

   return S_OK;
}

void CreateConnectionOption(TDataBrokerConnectionOptions *ptOpt)
{	
	//Set device IP
	printf("Please input device IP: ");
	gets(ptOpt->pzIPAddr);

	//Set device username
	printf("Please input device user name (press [ENTER] for empty): ");
	gets(ptOpt->pzUID);

	//Set device password
	printf("Please input device password (press [ENTER] for empty): ");
	gets(ptOpt->pzPWD);
	
	//Set device HTTP port
	printf("Please input device HTTP port: ");
	_tscanf("%hu", &(ptOpt->wHttpPort));

	//Set connection protocol type to TCP, you may refer to TsdrProtocolType for other protocols.
	ptOpt->dwProtocolType = eptTCP;

	//Set connection media type to Video and Audio.
	ptOpt->dwMediaType =  (emtVideo|emtAudio);

	//Set device server type to auto, the DataBroker will auto detect the server type.
	ptOpt->pzServerType = _T("auto");

	//Pass context to DataBrokerStatusCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwStatusContext = NULL;

	//Pass context to DataBrokerAVCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwAVContext = NULL;

	//Set status callback to connection option
	ptOpt->pfStatus = DataBrokerStatusCallBack;

	//Set Audio/Video callback to connection option
	ptOpt->pfAV = DataBrokerAVCallBack;

	//Set dwFlags to tell DataBroker what options we set.
	ptOpt->dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort | 
					 (eConOptStatusCallback | eConOptStatusContext) |
					 (eConOptAVCallback | eConOptAVContext);
}

int main()
{
	TCHAR szIP[MAX_IP_LENGTH] = {0};
	TCHAR szUserName[MAX_PASSWORD_LENGTH] = {0};
	TCHAR szPassword[MAX_USERNAME_LENGTH] = {0};
	
	SCODE scRet = S_FAIL;
	HANDLE	hDataBroker = NULL;
	HANDLE	hConn = NULL;
	int nMaxChannel = 32;
	DWORD dwStreamIndex = 0;
	
	// STEP 1: Initial DataBroker
	scRet = DataBroker_Initial(&hDataBroker, nMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, mctALLCODEC, 0, DATABROKER_VERSION);
	
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}

	
	// STEP 2: Create connection
	scRet = DataBroker_CreateConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 3: Set option to connection
	TDataBrokerConnectionOptions tOpt;
	memset(&tOpt, 0, sizeof(tOpt));
	tOpt.pzIPAddr = szIP;
	tOpt.pzUID = szUserName;
	tOpt.pzPWD = szPassword;

    CreateConnectionOption(&tOpt);	

	scRet = DataBroker_SetConnectionOptions(hConn, &tOpt);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}


    // STEP 4: Connect to device, this function is an asynchronous function, 
	//		   the connection information should be get in StatusCallback function we set.
	scRet = DataBroker_Connect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}

	printf("Press i key to force I frame\n");
	printf("Press s key to stop\n");

	char szbuffer[32] = {0};
	
	while (1)
	{
		gets(szbuffer);
		
		if (szbuffer[0] == 'i')
		{
			// STEP 5: Force the camera to send I-Frame 
			// Ssupport 7K/6K/3K series camera
			scRet = DataBroker_ForceIFrame(hConn);
			
			if (scRet != S_OK)
			{
				printf("DataBroker_ForceIFrame failed with error %X \n", scRet);
			}
		}
		if (szbuffer[0] == 's')
		{
			break;
		}
	}
		
	// STEP 6: Disconnect to device
	scRet = DataBroker_Disconnect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}

    // STEP 7: Delete connection
    scRet = DataBroker_DeleteConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 8: Release DataBroker
	scRet = DataBroker_Release(&hDataBroker);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}