#include <conio.h>
#include "tchar.h"
#include "DataBroker.h"
#include "SrvDepResource.h"

#define MAX_IP_LENGTH 256
#define MAX_USERNAME_LENGTH 256
#define MAX_PASSWORD_LENGTH 256

typedef struct {
	
	DWORD dwDINum;
	DWORD dwDONum;

} TDIDOInfo;

// This callback function is used to receive media packet, the calling convention must be stdcall
SCODE __stdcall DataBrokerAVCallBack(DWORD dwContext,TMediaDataPacketInfo *ptMediaDataPacket)
{
  
   return S_OK;
}

// This callback function is used to receive status info, the calling convention must be stdcall
SCODE __stdcall DataBrokerStatusCallBack(DWORD dwContext, TDataBrokerStatusType eStatusType, PVOID pvParam1, PVOID pvParam2)
{
	switch (eStatusType)
	{
		case eOnConnectionInfo:
		{
			printf("Get eOnConnectionInfo, start to receive packet\n");
		}
		break;

		case eOnStopped:
		{
			printf("Get eOnStopped, stop to receive packet\n");
		}
		break;

		case eOnDiDo:
		{
			//	STEP 1: Cast dwContext to TDIDOInfo*
			TDIDOInfo* ptDIDOInfo = (TDIDOInfo *) dwContext;

			//	STEP 2: Get the DI and DO value from pvParam2
			WORD wDIVal = LOWORD((DWORD) pvParam2);	//	The Low-word is represented the DI Value
			WORD wDOVal = HIWORD((DWORD) pvParam2); //	The High-word is represented the DO Value

			//	STEP 3: Parse DI related bits
			for (DWORD dwDIIdx = 0; dwDIIdx < ptDIDOInfo->dwDINum; dwDIIdx++)
			{
				//	Bit 0 for DI1, Bit 1 for DI2..., we use bit-operator to check the value.
				if (1 == (wDIVal & (1 << dwDIIdx)))
				{
					printf("DI-%d is HIGH \n", (dwDIIdx + 1));
				}
				else
				{
					printf("DI-%d is LOW \n", (dwDIIdx + 1));
				}
			}

			//	STEP 4: Parse DO related bits
			for (DWORD dwDOIdx = 0; dwDOIdx < ptDIDOInfo->dwDONum; dwDOIdx++)
			{
				//	Bit 0 for DO1, Bit 1 for DO2..., we use bit-operator to check the value.
				if (1 == (wDOVal & (1 << dwDOIdx)))
				{
					printf("DO-%d is HIGH\n", (dwDOIdx + 1));
				}
				else
				{
					printf("DO-%d is LOW\n", (dwDOIdx + 1));
				}
			}
		}
		break;

		default:
		{
		}	
		break;
	}

   return S_OK;
}

void CreateConnectionOption(TDataBrokerConnectionOptions *ptOpt, TDIDOInfo* ptDIDOInfo)
{	
	//Set device IP
	printf("Please input device IP: ");
	gets(ptOpt->pzIPAddr);

	//Set device username
	printf("Please input device user name (press [ENTER] for empty): ");
	gets(ptOpt->pzUID);

	//Set device password
	printf("Please input device password (press [ENTER] for empty): ");
	gets(ptOpt->pzPWD);

	//Set device HTTP port
	printf("Please input device HTTP port: ");
	_tscanf("%hu", &(ptOpt->wHttpPort));

	//Set DI Number, the DI number could be retrieved via ServerManager
	printf("Please input device DI Number: ");
	_tscanf("%d", &(ptDIDOInfo->dwDINum));

	//Set DO Number, the DO number could be retrieved via ServerManager
	printf("Please input device DO Number: ");
	_tscanf("%d", &(ptDIDOInfo->dwDONum));

	//Set connection protocol type to TCP, you may refer to TsdrProtocolType for other protocols.
	ptOpt->dwProtocolType = eptTCP;

	//Set connection media type to Video and Audio.
	ptOpt->dwMediaType =  (emtVideo | emtAudio);

	//Set device server type to auto, the DataBroker will auto detect the server type.
	ptOpt->pzServerType = _T("auto");

	//Pass DIDOInfo pointer address to DataBrokerStatusCallBack
	ptOpt->dwStatusContext = (DWORD) ptDIDOInfo;

	//Pass context to DataBrokerAVCallBack, the context is useful when we have to handle multi connections simultaneously.
	ptOpt->dwAVContext = NULL;

	//Set status callback to connection option
	ptOpt->pfStatus = DataBrokerStatusCallBack;

	//Set Audio/Video callback to connection option
	ptOpt->pfAV = DataBrokerAVCallBack;

	//Set dwFlags to tell DataBroker what options we set.
	ptOpt->dwFlags = eConOptProtocolAndMediaType | eConOptHttpPort | 
		(eConOptStatusCallback | eConOptStatusContext) |
		(eConOptAVCallback | eConOptAVContext);
}

void ParseDIDO(PVOID pvParam,  TDIDOInfo* ptDIDOInfo)
{
	
}


int main()
{
	TCHAR szIP[MAX_IP_LENGTH] = {0};
	TCHAR szUserName[MAX_PASSWORD_LENGTH] = {0};
	TCHAR szPassword[MAX_USERNAME_LENGTH] = {0};

	SCODE scRet = S_FAIL;
	HANDLE	hDataBroker = NULL;
	HANDLE	hConn = NULL;
	int nMaxChannel = 32;
	
	
	// STEP 1: Initial DataBroker
	scRet = DataBroker_Initial(&hDataBroker, nMaxChannel, DataBrokerStatusCallBack, DataBrokerAVCallBack, mctALLCODEC, 0, DATABROKER_VERSION);
	
	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Initial failed with error %X \n", scRet);
		return -1;
	}

	
	// STEP 2: Create connection
	scRet = DataBroker_CreateConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_CreateConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 3: Set option to connection
	TDataBrokerConnectionOptions tOpt;
	memset(&tOpt, 0, sizeof(tOpt));
	tOpt.pzIPAddr = szIP;
	tOpt.pzUID = szUserName;
	tOpt.pzPWD = szPassword;

	TDIDOInfo tDIDOInfo;
	memset(&tDIDOInfo, 0, sizeof(tDIDOInfo));
 
    CreateConnectionOption(&tOpt, &tDIDOInfo);               
	scRet = DataBroker_SetConnectionOptions(hConn, &tOpt);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_SetConnectionOptions failed with error %X \n", scRet);
		return -1;
	}

	// STEP 4: Connect to device, this function is an asynchronous function, 
	//		   the connection information should be get in StatusCallback function we set.
	scRet = DataBroker_Connect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Connect failed with error %X \n", scRet);
		return -1;
	}

	// Press any key to stop the receiving.
	while( !_kbhit() )
	{
		Sleep(16);
	}
		
	// STEP 5: Disconnect to device
	scRet = DataBroker_Disconnect(hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Disconnect failed with error %X \n", scRet);
		return -1;
	}

    // STEP 6: Delete connection
    scRet = DataBroker_DeleteConnection(hDataBroker, &hConn);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_DeleteConnection failed with error %X \n", scRet);
		return -1;
	}

	// STEP 7: Release DataBroker
	scRet = DataBroker_Release(&hDataBroker);

	if (IS_FAIL(scRet))
	{
		printf("DataBroker_Release failed with error %X \n", scRet);
		return -1;
	}

	return 0;
}